/*
 * Project name:
     Thermo2 click (Demonstration of mikroBUS THERMO2 click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150406:
       - initial release (DM);
 * Description:
     This code demonstrates how to use Thermo2 click with EasyPIC v7 board.
     Thermo2 sensor measures temperature.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL 32.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    Thermo2 click board click
                      http://www.mikroe.com/click/thermo2/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Place Thermo2 click board at the mikroBUS socket 1 on the EasyPIC v7 board.
     - Put LCD display in its socket, turn on backlight using the SW4.6
 */

#include "DS1825_Defs.h"
#include "DS1825_Driver.h"

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

char text[16];
extern scratchpad_t ScratchPad;
const float RESOLUTION_9 = 0.5;                                                 //9 bits
const float RESOLUTION_10 = 0.25;                                               //10 bits
const float RESOLUTION_11 = 0.125;                                              //11 bits
const float RESOLUTION_12 = 0.0625;                                             //12 bits

static void Display_Init(){
  LCD_Init();
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  // Display string on the LCD
  Lcd_Out(1,2,"THERMO2  click");
  Lcd_Out(2,3,"Demo Example");
  delay_ms(2000);
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Out(1,3,"Temperature:");
}

static void Display_Temperature() {
  char cnt;
  unsigned int temp_value;
  float temperature;

  temp_value = ScratchPad.Th << 8;
  temp_value = temp_value + ScratchPad.Tl;
  switch (ScratchPad.Resolution) {
    case 9:
      temperature = (float)temp_value * RESOLUTION_9;
      break;
    case 10:
      temperature = (float)temp_value * RESOLUTION_10;
      break;
    case 11:
      temperature = (float)temp_value * RESOLUTION_11;
      break;
    case 12:
      temperature = (float)temp_value * RESOLUTION_12;
      break;
    default:
      temperature = (float)temp_value * RESOLUTION_12;
  }
  sprintf(text, "%2.1f", temperature);
  Lcd_Out(2,6,text);
  Lcd_Chr(2,10,0xB2);
  Lcd_Chr(2,11,0x43);
}

void main() {
  Display_Init();
  DS1825_Init(12);
  delay_ms(100);
  while(1) {
    DS1825_StartConversion();
    DS1825_ReadScratchPad();
    Display_Temperature();
    delay_ms(500);
  }
}